--Copyright Tibor Karaszi Konsulting. Not to be used by other than Tibor Karaszi for educational purposes.

USE tempdb;

--En rad fr varje plan
SELECT * FROM sys.dm_exec_cached_plans
--Notera plan_handle

--Hmta batchen via dm_exec_sql_text()
SELECT
 cp.objtype AS PlanType
,OBJECT_NAME(t.objectid, t.dbid) AS ObjectName	--Namn p procedur, etc dr relevant
,cp.refcounts AS ReferenceCounts
,cp.usecounts AS UseCounts
,t.text AS SQLBatch
FROM sys.dm_exec_cached_plans AS cp
CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) AS t
--Hmm, ser hela batchen

--Statistik fr exekverade frgor
SELECT TOP(10) *
FROM sys.dm_exec_query_stats AS s
ORDER BY total_elapsed_time DESC


--Anvnder offsets i dm_exec_query_stats fr att plocka sjlva kommandot
SELECT TOP(10)
 s.execution_count
,s.total_elapsed_time
,s.total_logical_reads
,s.total_rows
,SUBSTRING(t.text, (s.statement_start_offset/2) + 1, ((CASE s.statement_end_offset WHEN -1 THEN DATALENGTH(t.text) ELSE s.statement_end_offset END - s.statement_start_offset)/2) + 1) as StatementText
FROM sys.dm_exec_query_stats AS s
 CROSS APPLY sys.dm_exec_sql_text(s.sql_handle) AS t
ORDER BY total_elapsed_time DESC


--Och s planen ocks via dm_exec_query_plan
SELECT TOP(10)
 s.execution_count
,s.total_elapsed_time
,s.total_logical_reads
,s.total_rows
,qp.query_plan
,SUBSTRING(t.text, (s.statement_start_offset/2) + 1, ((CASE s.statement_end_offset WHEN -1 THEN DATALENGTH(t.text) ELSE s.statement_end_offset END - s.statement_start_offset)/2) + 1) as StatementText
FROM sys.dm_exec_query_stats AS s
 CROSS APPLY sys.dm_exec_sql_text(s.sql_handle) AS t
 CROSS APPLY sys.dm_exec_query_plan(s.plan_handle) AS qp
ORDER BY total_elapsed_time DESC

--Fr tillfllet exekverande SQL kommandon
SELECT * FROM sys.dm_exec_requests

--Fr stored procedures
SELECT TOP(10)
 s.execution_count
,OBJECT_NAME(t.objectid, t.dbid) AS ObjectName	--Namn p procedur, etc dr relevant
,s.total_elapsed_time
,s.total_logical_reads
FROM sys.dm_exec_procedure_stats AS s
 CROSS APPLY sys.dm_exec_sql_text(s.sql_handle) AS t
CROSS APPLY sys.dm_exec_query_plan(s.plan_handle) AS qp
ORDER BY total_elapsed_time DESC

--Tm plan cachen
DBCC FREEPROCCACHE
GO


