--Copyright Tibor Karaszi Konsulting. Not to be used by other than Tibor Karaszi for educational purposes.

USE Adventureworks
GO

--------------------------------------------------------------------------------------------------------
--Erstt GetBlueProductsAndModels med en som tar frg som parameter
--------------------------------------------------------------------------------------------------------
CREATE OR ALTER PROC Production.GetProductsAndModelsByColor
  @Color nvarchar(15)
AS
SELECT p.ProductID,
     p.Name,
     p.Size,
     p.ListPrice,
	 p.Color
FROM Production.Product AS p
WHERE (p.Color = @Color) OR (p.Color IS NULL AND @Color IS NULL)
ORDER BY p.ProductID;

SELECT p.ProductID,
     pm.ProductModelID,
     pm.Name AS ModelName
FROM Production.Product AS p
INNER JOIN Production.ProductModel AS pm
ON p.ProductModelID = pm.ProductModelID 
WHERE (p.Color = @Color) OR (p.Color IS NULL AND @Color IS NULL)
ORDER BY p.ProductID, pm.ProductModelID;
GO

EXEC Production.GetProductsAndModelsByColor 'Red';
EXEC Production.GetProductsAndModelsByColor @color = 'Red';
EXEC Production.GetProductsAndModelsByColor @color = NULL;
GO

------------------------------------------------------------------------------------
--Output parm
------------------------------------------------------------------------------------
CREATE OR ALTER PROC p
 @pProdId int, @pProdName name OUT
AS
SET @pProdName = (SELECT name FROM Production.Product WHERE ProductID = @pProdId)
GO

DECLARE @vProdName name
EXEC p @pProdId = 4, @pProdName = @vProdName OUT
PRINT @vProdName
GO

--Alternativ
CREATE OR ALTER PROC p
 @pProdId int
AS
SELECT name FROM Production.Product WHERE ProductID = @pProdId
GO
EXEC p @pProdId = 4


------------------------------------------------------------------------------------


------------------------------------------------------------------------------------
--Parameter sniffing, etc
SET STATISTICS IO ON
EXEC sp_indexinfo 'Product'
CREATE INDEX x ON Production.Product(ListPrice)

--Utskrivet vrde (vrde knt), "literal"
SELECT * 
FROM Production.Product 
WHERE ListPrice > 5

SELECT * 
FROM Production.Product 
WHERE ListPrice > 5000

--Ovan kallas ibland ad-hoc (exakt text matching)
--"Plan cache bloat"
--Se optimize for ad-hoc workloads (cacha inte engngsplaner, sparar minne men inte CPU)


SELECT * FROM Production.Product

--Variabel (vrde oknt)
DECLARE @ListPrice money
SET @ListPrice = 5
SELECT * FROM Production.Product WHERE ListPrice > @ListPrice --OPTION(RECOMPILE)
GO



--Parameter (vrde sniffas, "parameter sniffing")
CREATE OR ALTER PROC p
 @ListPrice money
--WITH RECOMPILE
AS
--DECLARE @vListPrice money
--SET @vListPrice = @ListPrice
--SET @ListPrice = @ListPrice * 100

SELECT * 
FROM Production.Product 
WHERE ListPrice > @ListPrice
--OPTION(RECOMPILE)
--OPTION(OPTIMIZE FOR(@ListPrice = 23))
--OPTION(OPTIMIZE FOR UNKNOWN)
--RETURN -101
GO

SET STATISTICS TIME ON
SET STATISTICS IO ON
SET STATISTICS IO OFF
SET STATISTICS TIME OFF
EXEC p @ListPrice = 1
EXEC p @ListPrice = 5000

EXEC p @ListPrice = 1 WITH RECOMPILE
EXEC p @ListPrice = 5000 WITH RECOMPILE

DBCC FREEPROCCACHE
DBCC DROPCLEANBUFFERS



--Jobba parameteriserat via vrat API:
EXEC sp_executesql --parameter sniffing

EXEC sp_prepare --sniffar inte parameter, hanterar som en T-SQL @variabel
EXEC sp_execute

--Mer om ovan i https://www.sommarskog.se/query-plan-mysteries.html

-------------------------------------------------------------------
--Optional search parameters
--Se http://www.sommarskog.se/dyn-search.html


EXEC sp_indexinfo 'Product'
CREATE INDEX c ON Production.Product(Color)
CREATE INDEX s ON Production.Product(Size)
GO

DROP PROC p2
GO

CREATE PROC p2
 @Color nvarchar(20) 
,@Size nvarchar(20) = '42'
AS
SELECT * 
FROM Production.Product
WHERE ((Color = @Color) OR (@Color IS NULL))
AND ((Size = @Size) OR (@Size IS NULL))
OPTION(RECOMPILE)
GO

EXEC p2 'Blue', 'XL'
EXEC p2 NULL, 'XL'
EXEC p2 'Blue', NULL
EXEC p2 's', NULL

EXEC p2 @Size = 42, @Color = 'red'


--Stda
DROP PROC Production.GetProductsAndModelsByColor;

