--Copyright Tibor Karaszi Konsulting. Not to be used by other than Tibor Karaszi for educational purposes.

USE Adventureworks
GO
IF OBJECT_ID('CurrentPrice') IS NOT NULL DROP TABLE CurrentPrice
IF OBJECT_ID('PostalCode') IS NOT NULL DROP TABLE PostalCode
GO

------------------------------------------------------------------------------------------------------------------------------------
--"Soft delete" med INSTEAD OF DELETE trigger
------------------------------------------------------------------------------------------------------------------------------------
CREATE TABLE dbo.CurrentPrice
(
CurrentPriceID int IDENTITY(1,1) CONSTRAINT PK_CurrentPrice PRIMARY KEY,
SellingPrice decimal(18,2) NOT NULL,
LastModified datetime2(0) NOT NULL CONSTRAINT DF_CurrentPrice_LastModified DEFAULT(SYSDATETIME()),
ModifiedBy sysname NOT NULL CONSTRAINT DF_CurrentPrice_ModifiedBy DEFAULT(ORIGINAL_LOGIN()),
IsValid bit NOT NULL CONSTRAINT DF_CurrentPrice_IsValid DEFAULT(1)
);

INSERT INTO dbo.CurrentPrice  (SellingPrice)
VALUES (2.3), (4.3), (5);


SELECT * FROM dbo.CurrentPrice;
GO


--Skapa INSTEAD OF DELETE trigger
CREATE TRIGGER TR_CurrentPrice_Delete
ON dbo.CurrentPrice
INSTEAD OF DELETE AS 
  SET NOCOUNT ON;
  UPDATE CurrentPrice
  SET IsValid = 0
  WHERE CurrentPriceID IN(SELECT CurrentPriceID FROM deleted)
GO

--Frsk att ta bort rad, notera "soft delete"
DELETE dbo.CurrentPrice
WHERE CurrentPriceID = 1;

--Hur ser det ut?
SELECT * FROM dbo.CurrentPrice;
GO

--Meta-data
SELECT * FROM sys.triggers;
GO

--Stda
DROP TABLE dbo.CurrentPrice;








------------------------------------------------------------------------------------------------------------------------------------
--Gr vy uppdaterbar
------------------------------------------------------------------------------------------------------------------------------------

CREATE TABLE dbo.PostalCode
( CustomerID int PRIMARY KEY,
  PostalCode nvarchar(5) NOT NULL,
  PostalSubCode nvarchar(5) NULL
);
GO


--Vy som slr ihop strngar
CREATE VIEW dbo.PostalRegion
AS
SELECT CustomerID,
       PostalCode + COALESCE('-' + PostalSubCode,'') AS PostalRegion
FROM dbo.PostalCode;
GO

INSERT dbo.PostalCode (CustomerID,PostalCode,PostalSubCode)
VALUES (1,'23422','234'),
       (2,'23523',NULL),
       (3,'08022','523');

SELECT * FROM dbo.PostalCode
SELECT * FROM dbo.PostalRegion;

--Frsk att gra INSERT till vy
INSERT INTO dbo.PostalRegion (CustomerID,PostalRegion)
VALUES (4,'09232-432');

--Frsk att gra UPDATE mot vy
UPDATE dbo.PostalRegion SET PostalRegion = '23234-523' WHERE CustomerID = 3;

--Frsk att gra DELETE frn vy
DELETE FROM dbo.PostalRegion WHERE CustomerID = 3;
GO
--Varfr gick det bra?

--INSTEAD OF trigger p vy
CREATE TRIGGER IX_PostalRegion_Insert
ON dbo.PostalRegion
INSTEAD OF INSERT
AS
SET NOCOUNT ON
INSERT INTO dbo.PostalCode (CustomerID ,  PostalCode ,  PostalSubCode)
SELECT CustomerID, 
       SUBSTRING(PostalRegion,1,5),
       CASE WHEN SUBSTRING(PostalRegion,7,5) <> '' THEN SUBSTRING(PostalRegion,7,5) END
FROM inserted;
GO

--Prova igen
INSERT INTO dbo.PostalRegion (CustomerID,PostalRegion)
VALUES (4,'09232-432');

SELECT * FROM dbo.PostalCode
SELECT * FROM dbo.PostalRegion;
GO

--Fixa ROWCOUNT meddelanden
ALTER TRIGGER IX_PostalRegion_Insert
ON dbo.PostalRegion
INSTEAD OF INSERT
AS
SET NOCOUNT ON;
INSERT INTO dbo.PostalCode 
SELECT CustomerID, 
       SUBSTRING(PostalRegion,1,5),
       CASE WHEN SUBSTRING(PostalRegion,7,5) <> '' THEN SUBSTRING(PostalRegion,7,5) END
FROM inserted;
GO

--Bttre?
INSERT INTO dbo.PostalRegion (CustomerID,PostalRegion)
VALUES (5,'92232-142');

SELECT * FROM dbo.PostalRegion;

--Verifiera att triggern funkar fr modifikation av flera rader
INSERT INTO dbo.PostalRegion (CustomerID,PostalRegion)
VALUES (6,'11111-111'),
       (7,'99999-999');

SELECT * FROM dbo.PostalRegion;
SELECT * FROM dbo.PostalCode

--Stda
DROP VIEW dbo.PostalRegion;
GO
DROP TABLE dbo.PostalCode;
